package com.inteligr8.alfresco.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation tells the framework to execute the annotated method
 * inside an ACS authorized context.  This is how the execution can be elevated
 * to a service account or de-escalated to a user account.
 * 
 * If the authorization is expected to be the same (the same user), then
 * another layer of authorization is **not** added.
 * 
 * Use the Authorizable interface to provide a dynamic user ID for the
 * authorization context.
 * 
 * @see com.inteligr8.alfresco.annotations.Authorizable
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Authorized {
	
	/**
	 * The user ID to use for the authorization context.
	 * 
	 * It is important to note that if the Authorizable interface is
	 * implemented, then the value returned from its method will take
	 * precedence over this one.  This capability is useful for to support
	 * dynamic user authorization contexts.  
	 * 
	 * @return An ACS user ID; empty will be treated as `system`.
	 */
	String value() default "";

}
