package com.inteligr8.alfresco.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Repeatable;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation tells the framework to only execute the annotated method if
 * the ACS authenticated user is authorized.
 * 
 * @see com.inteligr8.alfresco.annotations.Authorizable
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
@Repeatable(IfAuthorizeds.class)
public @interface IfAuthorized {
	
	/**
	 * The authorities (users and/or user groups) to constrain for the
	 * authorization context.  Only one authority needs to match.  To
	 * require multiple authorities, use multiple @IfAuthorized
	 * annotations.
	 * 
	 * @return An array of ACS authorities; empty means any authenticated user.
	 */
	String[] value() default "";

}
