package com.inteligr8.alfresco.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation tells the framework to skip execution if the annotated
 * parameter or any annotated method parameter is a node reference and it does
 * not have the specified aspect.  Unless cached, this will result in the
 * framework consulting with the database.
 * 
 * This includes support for checking the child node reference of a
 * parent-child association and both the source/target of a peer association.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({
	ElementType.METHOD,
	ElementType.PARAMETER
})
public @interface IfNodeHasAspect {
	
	/**
	 * @return An ACS aspect in the Alfresco QName prefixed format (e.g. `cm:auditable`).
	 */
	String aspect() default "";

}
