package com.inteligr8.alfresco.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation tells the framework to skip execution if the annotated
 * parameter or any annotated method parameter is a node reference and it is
 * not of the specified type.  Unless cached, this will result in the framework
 * consulting with the database.
 * 
 * This includes support for checking the child node reference of a
 * parent-child association and both the source/target of a peer association.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({
	ElementType.METHOD,
	ElementType.PARAMETER
})
public @interface IfNodeOfType {
	
	/**
	 * @return An ACS node type in the Alfresco QName prefixed format (e.g. `cm:content`).
	 */
	String type() default "";

}
