package com.inteligr8.alfresco.annotations;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.alfresco.service.namespace.NamespaceService;
import org.alfresco.service.namespace.QName;
import org.alfresco.service.namespace.QNamePattern;
import org.alfresco.service.namespace.RegexQNamePattern;
import org.springframework.beans.factory.BeanNameAware;

public interface NodeAspectConstrainable extends BeanNameAware {
	
	String getBeanName();
	
	NamespaceService getNamespaceService();
	
	default Collection<String> constrainedPrefixedAspects() {
		return Collections.emptySet();
	}
	
	default Collection<String> constrainedRegexedAspects() {
		return Collections.emptySet();
	}
	
	default Collection<? extends QNamePattern> constrainedAspects() {
		Set<QNamePattern> aspects = new HashSet<>();
		
		Collection<String> prefixedAspects = this.constrainedPrefixedAspects();
		for (String prefixedAspect : prefixedAspects)
			aspects.add(QName.createQName(prefixedAspect, this.getNamespaceService()));

		Collection<String> regexedAspects = this.constrainedRegexedAspects();
		for (String regexedAspect : regexedAspects)
			aspects.add(new RegexQNamePattern(regexedAspect));
		
		return aspects;
	}

}
