package com.inteligr8.alfresco.annotations;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.alfresco.service.namespace.NamespaceService;
import org.alfresco.service.namespace.QName;
import org.alfresco.service.namespace.QNamePattern;
import org.alfresco.service.namespace.RegexQNamePattern;
import org.springframework.beans.factory.BeanNameAware;

public interface NodeTypeConstrainable extends BeanNameAware {
	
	String getBeanName();
	
	NamespaceService getNamespaceService();
	
	default Collection<String> constrainedPrefixedNodeTypes() {
		return Collections.emptySet();
	}
	
	default Collection<String> constrainedRegexedNodeTypes() {
		return Collections.emptySet();
	}
	
	default Collection<? extends QNamePattern> constrainedNodeTypes() {
		Set<QNamePattern> nodeTypes = new HashSet<>();
		
		Collection<String> prefixedNodeTypes = this.constrainedPrefixedNodeTypes();
		for (String prefixedNodeType : prefixedNodeTypes)
			nodeTypes.add(QName.createQName(prefixedNodeType, this.getNamespaceService()));

		Collection<String> regexedNodeTypes = this.constrainedRegexedNodeTypes();
		for (String regexedNodeType : regexedNodeTypes)
			nodeTypes.add(new RegexQNamePattern(regexedNodeType));
		
		return nodeTypes;
	}

}
