package com.inteligr8.alfresco.annotations.aspect;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractMethodOrParameterAspect<T extends Annotation> {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	public abstract Class<T> getAnnotationClass();
	
	public Object checkParameters(ProceedingJoinPoint joinPoint, ApplicableParameterCallback<T> callback) throws Throwable {
		MethodSignature methodSig = (MethodSignature) joinPoint.getSignature();
		Method method = methodSig.getMethod();
		T methodAnnotation = method.getAnnotation(this.getAnnotationClass());
		
		for (int p = 0; p < method.getParameterCount(); p++) {
			T annotation = method.getParameters()[p].getAnnotation(this.getAnnotationClass());
			if (annotation == null)
				annotation = methodAnnotation;
			if (annotation != null) {
				if (!callback.checkParameter(joinPoint, method, annotation, p)) {
					this.logger.debug("The parameter '{}' condition is false; skipping method: {}", method.getParameters()[p].getName(), method);
					return null;
				}
			}
		}
		
		return joinPoint.proceed();
	}

}
