package com.inteligr8.alfresco.annotations.aspect;

import java.lang.reflect.Method;

import org.alfresco.service.cmr.repository.ChildAssociationRef;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.inteligr8.alfresco.annotations.IfChildAssociationIsPrimary;

/**
 * This aspect implements the IfChildAssociationIsPrimary annotation.
 * 
 * @see com.inteligr8.alfresco.annotations.IfChildAssociationIsPrimary
 */
@Aspect
public class ChildIsPrimaryAspect extends AbstractMethodOrParameterAspect<IfChildAssociationIsPrimary> {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Override
	public Class<IfChildAssociationIsPrimary> getAnnotationClass() {
		return IfChildAssociationIsPrimary.class;
	}
	
	@Pointcut("@annotation(com.inteligr8.alfresco.annotations.IfChildAssociationIsPrimary) && execution(* *(..))")
	public void isIfChildAssociationIsPrimaryMethodAnnotated() {
	}

	@Pointcut("execution(* *(@com.inteligr8.alfresco.annotations.IfChildAssociationIsPrimary (*), ..))")
	public void isIfChildAssociationIsPrimaryParamAnnotated() {
	}
	
	@Around("isIfChildAssociationIsPrimaryMethodAnnotated() || isIfChildAssociationIsPrimaryParamAnnotated()")
	public Object isChildAssocPrimary(ProceedingJoinPoint joinPoint) throws Throwable {
		this.logger.trace("isChildAssocPrimary({})", joinPoint);
		
		return this.checkParameters(joinPoint, new ApplicableParameterCallback<IfChildAssociationIsPrimary>() {
			@Override
			public boolean checkParameter(ProceedingJoinPoint joinPoint, Method method, IfChildAssociationIsPrimary annotation, int parameterIndex) {
				Object arg = joinPoint.getArgs()[parameterIndex];
				if (arg instanceof ChildAssociationRef) {
					if (!((ChildAssociationRef)arg).isPrimary()) {
						logger.debug("The child association '{}' is not primary; skipping method: {}", arg, method);
						return false;
					}
				}
				
				return true;
			}
		});
	}

}
