package com.inteligr8.alfresco.annotations.aspect;

import java.lang.reflect.Method;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.inteligr8.alfresco.annotations.IfNotNull;

/**
 * This aspect implements the IfNotNull annotation.
 * 
 * @see com.inteligr8.alfresco.annotations.IfNotNull
 */
@Aspect
public class NotNullAspect {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Pointcut("execution(* *(@com.inteligr8.alfresco.annotations.IfNotNull (*), ..))")
	public void isNotNullAnnotated() {
	}
	
	@Around("isNotNullAnnotated()")
	public Object isNotNull(ProceedingJoinPoint joinPoint) throws Throwable {
		MethodSignature methodSig = (MethodSignature) joinPoint.getSignature();
		Method method = methodSig.getMethod();
		
		for (int p = 0; p < method.getParameterCount(); p++) {
			if (joinPoint.getArgs()[p] == null && method.getParameters()[p].isAnnotationPresent(IfNotNull.class)) {
				this.logger.debug("A @IfNotNull parameter is `null`; skipping method: {}", method);
				return null;
			}
		}
		
		return joinPoint.proceed();
	}

}
