package com.inteligr8.alfresco.annotations.util;

import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Component
@Scope(scopeName = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class SpringTransactionalAnnotationAdapter implements TransactionalAnnotationAdapter {
	
	private final Transactional txl;
	
	public SpringTransactionalAnnotationAdapter(Transactional txl) {
		this.txl = txl;
	}

	@Override
	public boolean isReadOnly() {
		return this.txl.readOnly();
	}

	@Override
	public Propagation getPropagation() {
		return this.txl.propagation();
	}

	@Override
	public Isolation getIsolation() {
		return this.txl.isolation();
	}

	@Override
	public int getTimeoutInSeconds() {
		return this.txl.timeout();
	}

	@Override
	public Class<? extends Throwable>[] getRollbackFor() {
		return this.txl.rollbackFor();
	}

	@Override
	public Class<? extends Throwable>[] getNoRollbackFor() {
		return this.txl.noRollbackFor();
	}

}
