package com.inteligr8.alfresco.annotations;

import org.alfresco.error.AlfrescoRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class AsynchronousTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private long mainThreadId;
	private Object obj;
	
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());

		this.mainThreadId = Thread.currentThread().getId();
		this.logger.info("Running in thread: {}", this.mainThreadId);
		
		this.tryThreadPool();
		this.tryMq();
		
		try {
			Thread.sleep(500L);
		} catch (InterruptedException ie) {
			throw new AlfrescoRuntimeException("This should never happen", ie);
		}
		
		this.obj = new Object();
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@Asynchronous(durable = false)
	private void tryThreadPool() {
		this.logger.info("Running in another thread: {}", Thread.currentThread().getId());
		
		Assert.isTrue(Thread.currentThread().getId() != this.mainThreadId, "This method is not executing in a separate thread");
		
		Assert.isNull(this.obj, "The random value is expected to be 'null'");

		try {
			Thread.sleep(1000L);
		} catch (InterruptedException ie) {
			throw new AlfrescoRuntimeException("This should never happen", ie);
		}
		
		Assert.isTrue(this.obj != null, "The random value is not expected to be 'null'");
	}
	
	@Asynchronous
	private void tryMq() {
		this.logger.info("Running in another thread: {}", Thread.currentThread().getId());
		
		Assert.isTrue(Thread.currentThread().getId() != this.mainThreadId, "This method is not executing in a separate thread");
		
		Assert.isNull(this.obj, "The random value is expected to be 'null'");

		try {
			Thread.sleep(1000L);
		} catch (InterruptedException ie) {
			throw new AlfrescoRuntimeException("This should never happen", ie);
		}
		
		Assert.isTrue(this.obj != null, "The random value is not expected to be 'null'");
	}

}
