package com.inteligr8.alfresco.annotations;

import java.util.UUID;

import org.alfresco.model.ContentModel;
import org.alfresco.service.cmr.repository.ChildAssociationRef;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.StoreRef;
import org.alfresco.service.namespace.NamespaceService;
import org.alfresco.service.namespace.QName;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class IfChildAssocIsPrimaryTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private final MutableBoolean executed = new MutableBoolean();
	
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());
		
		NodeRef mockNodeRef = new NodeRef(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE, UUID.randomUUID().toString());
		QName mockName = QName.createQNameWithValidLocalName(NamespaceService.ALFRESCO_URI, "test");
		ChildAssociationRef mockPrimaryAssoc = new ChildAssociationRef(ContentModel.ASSOC_CONTAINS, mockNodeRef, mockName, mockNodeRef, true, 0);
		ChildAssociationRef mockSecondaryAssoc = new ChildAssociationRef(ContentModel.ASSOC_CONTAINS, mockNodeRef, mockName, mockNodeRef, false, 0);
		
		this.executed.setFalse();
		this.tryMethodNoParam();
		if (this.executed.isFalse())
			throw new IllegalArgumentException();

		this.executed.setFalse();
		this.tryMethodNoMatchingParam("test");
		if (this.executed.isFalse())
			throw new IllegalArgumentException();
		
		this.tryMethodWithOnePrimaryParam(mockPrimaryAssoc);
		this.tryMethodWithOneNullParam(null);
		this.tryMethodWithOneNonPrimaryParam(mockSecondaryAssoc);
		
		this.tryMethodWithTwoPrimaryParams(mockPrimaryAssoc, "test", mockPrimaryAssoc);
		this.tryMethodWithTwoNullPrimaryParams(null, "test", mockPrimaryAssoc);
		this.tryMethodWithTwoNullPrimaryParams(mockPrimaryAssoc, "test", null);
		this.tryMethodWithTwoMixedParams(mockPrimaryAssoc, "test", mockSecondaryAssoc);
		this.tryMethodWithTwoMixedParams(mockSecondaryAssoc, "test", mockSecondaryAssoc);

		this.tryParamWithPrimaryParam(mockPrimaryAssoc, mockPrimaryAssoc);
		this.tryParamWithPrimaryParam(mockPrimaryAssoc, mockSecondaryAssoc);
		this.tryParamWithPrimaryParam(mockSecondaryAssoc, mockPrimaryAssoc);
		this.tryParamWithNullParam(null, mockPrimaryAssoc);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodNoParam() {
		this.executed.setTrue();
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodNoMatchingParam(String test) {
		this.executed.setTrue();
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodWithOnePrimaryParam(ChildAssociationRef childAssocRef) {
		Assert.isTrue(childAssocRef.isPrimary(), "Unexpected non-primary child association");
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodWithOneNullParam(ChildAssociationRef childAssocRef) {
		Assert.isTrue(childAssocRef == null, "Unexpected child association");
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodWithOneNonPrimaryParam(ChildAssociationRef childAssocRef) {
		throw new UnsupportedOperationException();
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodWithTwoPrimaryParams(ChildAssociationRef childAssocRef1, String test, ChildAssociationRef childAssocRef2) {
		Assert.isTrue(childAssocRef1.isPrimary() && childAssocRef2.isPrimary(), "Unexpected non-primary child association");
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodWithTwoNullPrimaryParams(ChildAssociationRef childAssocRef1, String test, ChildAssociationRef childAssocRef2) {
		Assert.isTrue(childAssocRef1 == null || childAssocRef1.isPrimary(), "Unexpected non-primary child association");
		Assert.isTrue(childAssocRef2 == null || childAssocRef2.isPrimary(), "Unexpected non-primary child association");
	}
	
	@IfChildAssociationIsPrimary
	private void tryMethodWithTwoMixedParams(ChildAssociationRef childAssocRef1, String test, ChildAssociationRef childAssocRef2) {
		throw new UnsupportedOperationException();
	}
	
	private void tryParamWithPrimaryParam(@IfChildAssociationIsPrimary ChildAssociationRef childAssocRef1, ChildAssociationRef childAssocRef2) {
		Assert.isTrue(childAssocRef1.isPrimary(), "Unexpected non-primary child association");
	}
	
	private void tryParamWithNullParam(@IfChildAssociationIsPrimary ChildAssociationRef childAssocRef1, ChildAssociationRef childAssocRef2) {
		Assert.isTrue(childAssocRef1 == null, "Unexpected child association");
	}

}
