package com.inteligr8.alfresco.annotations;

import java.util.Arrays;
import java.util.Collection;

import org.alfresco.model.ContentModel;
import org.alfresco.repo.nodelocator.CompanyHomeNodeLocator;
import org.alfresco.repo.nodelocator.NodeLocatorService;
import org.alfresco.repo.nodelocator.SharedHomeNodeLocator;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.cmr.repository.StoreRef;
import org.alfresco.service.namespace.NamespaceService;
import org.alfresco.service.namespace.QNamePattern;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class IfNodeAspectConstrainableTest extends AbstractLifecycleBean implements NodeAspectConstrainable {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	private NamespaceService namespaceService;
	
	@Autowired
	private NodeService nodeService;
	
	@Autowired
	private NodeLocatorService nodeLocatorService;
	
	private final MutableBoolean executed = new MutableBoolean();
	
	private String beanName;
	
	@AuthorizedAsSystem
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());

		NodeRef rootNodeRef = this.nodeService.getRootNode(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE);
		NodeRef chNodeRef = this.nodeLocatorService.getNode(CompanyHomeNodeLocator.NAME, null, null);
		NodeRef sharedNodeRef = this.nodeLocatorService.getNode(SharedHomeNodeLocator.NAME, null, null);
		
		this.tryNodes(chNodeRef, sharedNodeRef);
		this.tryNode(sharedNodeRef);

		this.executed.setFalse();
		this.tryNull(null);
		if (this.executed.isFalse())
			throw new IllegalStateException();
		
		this.tryNotNode(rootNodeRef);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@Override
	public String getBeanName() {
		return this.beanName;
	}
	
	@Override
	public void setBeanName(String name) {
		this.beanName = name;
	}
	
	@Override
	public NamespaceService getNamespaceService() {
		return this.namespaceService;
	}
	
	@Override
	public Collection<? extends QNamePattern> constrainedAspects() {
		return Arrays.asList(ContentModel.ASPECT_AUDITABLE);
	}

	@IfNodeHasAspect
	private void tryNodes(NodeRef... nodeRefs) {
		for (NodeRef nodeRef : nodeRefs) {
			Assert.isTrue(this.nodeService.hasAspect(nodeRef, ContentModel.ASPECT_AUDITABLE), "Missing 'cm:auditable' aspect: " + nodeRef);
		}
	}

	private void tryNode(@IfNodeHasAspect NodeRef nodeRef) {
		Assert.isTrue(this.nodeService.hasAspect(nodeRef, ContentModel.ASPECT_AUDITABLE), "Missing 'cm:auditable' aspect: " + nodeRef);
	}

	@IfNodeHasAspect
	private void tryNotNodes(NodeRef... nodeRefs) {
		throw new UnsupportedOperationException();
	}
	
	@IfNodeHasAspect
	private void tryNull(NodeRef nodeRef) {
		this.executed.setTrue();
	}

	private void tryNotNode(@IfNodeHasAspect NodeRef nodeRef) {
		throw new UnsupportedOperationException();
	}

}
