package com.inteligr8.alfresco.annotations;

import org.alfresco.model.ContentModel;
import org.alfresco.repo.nodelocator.NodeLocatorService;
import org.alfresco.repo.nodelocator.SharedHomeNodeLocator;
import org.alfresco.service.cmr.repository.ChildAssociationRef;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.cmr.repository.StoreRef;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class IfNodeAspectTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	private NodeService nodeService;
	
	@Autowired
	private NodeLocatorService nodeLocatorService;
	
	private final MutableBoolean executed = new MutableBoolean();

	@AuthorizedAsSystem
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());

		NodeRef rootNodeRef = this.nodeService.getRootNode(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE);
		NodeRef sharedNodeRef = this.nodeLocatorService.getNode(SharedHomeNodeLocator.NAME, null, null);
		ChildAssociationRef sharedParentRef = this.nodeService.getPrimaryParent(sharedNodeRef);
		
		this.tryAuditables(sharedNodeRef);
		this.tryAuditable(sharedNodeRef);
		this.tryAuditable(sharedParentRef);

		this.executed.setFalse();
		this.tryNull(null);
		if (this.executed.isFalse())
			throw new IllegalStateException();
		
		this.tryNotAuditable(rootNodeRef);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}

	@IfNodeHasAspect(aspect = "cm:auditable")
	private void tryAuditables(NodeRef... nodeRefs) {
		for (NodeRef nodeRef : nodeRefs) {
			Assert.isTrue(this.nodeService.hasAspect(nodeRef, ContentModel.ASPECT_AUDITABLE), "Missing 'cm:auditable' aspect: " + nodeRef);
		}
	}
	
	private void tryAuditable(@IfNodeHasAspect(aspect = "cm:auditable") NodeRef nodeRef) {
		Assert.isTrue(this.nodeService.hasAspect(nodeRef, ContentModel.ASPECT_AUDITABLE), "Missing 'cm:auditable' aspect: " + nodeRef);
	}
	
	private void tryAuditable(@IfNodeHasAspect(aspect = "cm:auditable") ChildAssociationRef childAssocRef) {
		NodeRef nodeRef = childAssocRef.getChildRef();
		Assert.isTrue(this.nodeService.hasAspect(nodeRef, ContentModel.ASPECT_AUDITABLE), "Missing 'cm:auditable' aspect: " + nodeRef);
	}

	@IfNodeHasAspect(aspect = "cm:auditable")
	private void tryNotAuditables(NodeRef... nodeRefs) {
		throw new UnsupportedOperationException();
	}
	
	@IfNodeHasAspect(aspect = "cm:auditable")
	private void tryNull(NodeRef nodeRef) {
		this.executed.setTrue();
	}

	private void tryNotAuditable(@IfNodeOfType(type = "cm:auditable") NodeRef nodeRef) {
		throw new UnsupportedOperationException();
	}

}
