package com.inteligr8.alfresco.annotations;

import java.util.UUID;

import org.alfresco.repo.nodelocator.CompanyHomeNodeLocator;
import org.alfresco.repo.nodelocator.NodeLocatorService;
import org.alfresco.repo.nodelocator.SharedHomeNodeLocator;
import org.alfresco.repo.nodelocator.SitesHomeNodeLocator;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.cmr.repository.StoreRef;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class IfNodeExistsTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	private NodeService nodeService;
	
	@Autowired
	private NodeLocatorService nodeLocatorService;
	
	private final MutableBoolean executed = new MutableBoolean();
	
	@AuthorizedAsSystem
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());

		NodeRef mockNodeRef = new NodeRef(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE, UUID.randomUUID().toString());
		NodeRef rootNodeRef = this.nodeLocatorService.getNode(CompanyHomeNodeLocator.NAME, null, null);
		NodeRef sharedNodeRef = this.nodeLocatorService.getNode(SharedHomeNodeLocator.NAME, null, null);
		NodeRef sitesNodeRef = this.nodeLocatorService.getNode(SitesHomeNodeLocator.NAME, null, null);
		
		this.tryNodesExist(rootNodeRef, sharedNodeRef, sitesNodeRef);
		this.tryNodeExists(sharedNodeRef);
		
		this.executed.setFalse();
		this.tryNull(null);
		if (this.executed.isFalse())
			throw new IllegalStateException();
		
		this.tryNodesNotExist(mockNodeRef);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@IfNodeExists
	private void tryNodesExist(NodeRef... nodeRefs) {
		for (NodeRef nodeRef : nodeRefs) {
			Assert.isTrue(nodeRef != null, "Expected node reference: " + nodeRef);
			Assert.isTrue(this.nodeService.exists(nodeRef) && !this.nodeService.getNodeStatus(nodeRef).isDeleted(), "Expected node: " + nodeRef);
		}
	}
	
	private void tryNodeExists(@IfNodeExists NodeRef nodeRef) {
		Assert.isTrue(nodeRef != null, "Expected node reference: " + nodeRef);
		Assert.isTrue(this.nodeService.exists(nodeRef) && !this.nodeService.getNodeStatus(nodeRef).isDeleted(), "Expected node: " + nodeRef);
	}
	
	@IfNodeExists
	private void tryNull(NodeRef nodeRef) {
		this.executed.setTrue();
	}
	
	@IfNodeExists
	private void tryNodesNotExist(NodeRef... nodeRefs) {
		throw new IllegalStateException();
	}

}
