package com.inteligr8.alfresco.annotations;

import org.alfresco.model.ContentModel;
import org.alfresco.repo.nodelocator.CompanyHomeNodeLocator;
import org.alfresco.repo.nodelocator.NodeLocatorService;
import org.alfresco.repo.nodelocator.SharedHomeNodeLocator;
import org.alfresco.service.cmr.dictionary.DictionaryService;
import org.alfresco.service.cmr.repository.ChildAssociationRef;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.cmr.repository.StoreRef;
import org.alfresco.service.namespace.QName;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class IfNodeTypeTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	private DictionaryService dictionaryService;
	
	@Autowired
	private NodeService nodeService;
	
	@Autowired
	private NodeLocatorService nodeLocatorService;
	
	private final MutableBoolean executed = new MutableBoolean();

	@AuthorizedAsSystem
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());

		NodeRef rootNodeRef = this.nodeService.getRootNode(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE);
		NodeRef chNodeRef = this.nodeLocatorService.getNode(CompanyHomeNodeLocator.NAME, null, null);
		NodeRef sharedNodeRef = this.nodeLocatorService.getNode(SharedHomeNodeLocator.NAME, null, null);
		ChildAssociationRef sharedParentRef = this.nodeService.getPrimaryParent(sharedNodeRef);
		
		this.tryFolders(chNodeRef, sharedNodeRef);
		this.tryFolder(sharedNodeRef);
		this.tryFolder(sharedParentRef);

		this.executed.setFalse();
		this.tryNull(null);
		if (this.executed.isFalse())
			throw new IllegalStateException();
		
		this.tryNotFolder(rootNodeRef);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}

	@IfNodeOfType(type = "cm:folder")
	private void tryFolders(NodeRef... nodeRefs) {
		for (NodeRef nodeRef : nodeRefs) {
			QName nodeType = this.nodeService.getType(nodeRef);
			Assert.isTrue(this.dictionaryService.isSubClass(nodeType, ContentModel.TYPE_FOLDER), "Unexpected node type: " + nodeType);
		}
	}
	
	private void tryFolder(@IfNodeOfType(type = "cm:folder") NodeRef nodeRef) {
		QName nodeType = this.nodeService.getType(nodeRef);
		Assert.isTrue(this.dictionaryService.isSubClass(nodeType, ContentModel.TYPE_FOLDER), "Unexpected node type: " + nodeType);
	}
	
	private void tryFolder(@IfNodeOfType(type = "cm:folder") ChildAssociationRef childAssocRef) {
		QName nodeType = this.nodeService.getType(childAssocRef.getChildRef());
		Assert.isTrue(this.dictionaryService.isSubClass(nodeType, ContentModel.TYPE_FOLDER), "Unexpected node type: " + nodeType);
	}

	@IfNodeOfType(type = "cm:folder")
	private void tryNotFolders(NodeRef... nodeRefs) {
		throw new UnsupportedOperationException();
	}
	
	@IfNodeOfType
	private void tryNull(NodeRef nodeRef) {
		this.executed.setTrue();
	}

	private void tryNotFolder(@IfNodeOfType(type = "cm:folder") NodeRef nodeRef) {
		throw new UnsupportedOperationException();
	}

}
