package com.inteligr8.alfresco.annotations;

import org.apache.commons.lang3.mutable.MutableInt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class JobSynchronizedTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		MutableInt threadsRun = new MutableInt();
		this.simpleLock();
		this.threadThenLock(threadsRun);
		this.threadAndLock(threadsRun);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@JobSynchronized
	private void simpleLock() {
		this.logger.debug("simpleLock()");
	}
	
	@Threaded(name = "job-sync", threads = 5, join = true)
	@JobSynchronized
	private void threadAndLock(MutableInt threadsRun) {
		this.lock(threadsRun);
	}
	
	@Threaded(name = "job-sync", threads = 5, join = true)
	private void threadThenLock(MutableInt threadsRun) {
		this.lock(threadsRun);
	}

	@JobSynchronized
	private void lock(MutableInt threadsRun) {
		this.locked(threadsRun);
	}

	private void locked(MutableInt threadsRun) {
		int t = threadsRun.intValue();
		this.logger.debug("After start of a mutually exclusive execution block: {}", t);
		
		try {
			Thread.sleep(100L);
		} catch (InterruptedException ie) {
		}

		Assert.isTrue(t == threadsRun.intValue(), "The threads run unexpectedly changed: " + t + " != " + threadsRun);
		threadsRun.increment();
		this.logger.debug("Before end of a mutually exclusive execution block: {}", t);
	}

}
