package com.inteligr8.alfresco.annotations;

import org.alfresco.repo.security.authentication.AuthenticationUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class NoExistAuthorizableTest extends AbstractLifecycleBean implements Authorizable {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: " + this.getClass());
		Assert.isNull(AuthenticationUtil.getRunAsUser(), "An unexpected authorization: " + AuthenticationUtil.getRunAsUser());
		
		this.tryAuthorized();
		this.tryAuthorizedAsSystem();
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@Override
	public String authorizeAsUser() {
		return "doesnotexist";
	}
	
	@Authorized
	private void tryAuthorized() {
		Assert.hasText(AuthenticationUtil.getRunAsUser(), "An expected authorization");
		Assert.isTrue(this.authorizeAsUser().equals(AuthenticationUtil.getRunAsUser()), "An unexpected authorization: " + this.authorizeAsUser() + " != " + AuthenticationUtil.getRunAsUser());
	}
	
	@AuthorizedAsSystem
	private void tryAuthorizedAsSystem() {
		Assert.hasText(AuthenticationUtil.getRunAsUser(), "An expected authorization");
		Assert.isTrue(AuthenticationUtil.getSystemUserName().equals(AuthenticationUtil.getRunAsUser()), "An unexpected authorization: " + AuthenticationUtil.getSystemUserName() + " != " + AuthenticationUtil.getRunAsUser());
	}

}
