package com.inteligr8.alfresco.annotations;

import java.util.Random;

import org.apache.commons.lang3.mutable.MutableInt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

@Component
public class ThreadedTest extends AbstractLifecycleBean {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private final Random random = new Random();
	
	@Override
	protected void onBootstrap(ApplicationEvent event) {
		this.logger.info("Running test: {}", this.getClass());
		Assert.isTrue(Thread.currentThread().getId() > 0L, "An unexpected non-positive thread ID: " + Thread.currentThread().getId());

		this.logger.info("Main thread: {}", Thread.currentThread().getId());
		
		MutableInt threadCount = new MutableInt();
		this.try5sync(Thread.currentThread(), threadCount);
		Assert.isTrue(threadCount.intValue() == 5, "An unexpected thread count record: 5 != " + threadCount);

		threadCount.setValue(0);
		this.try5(Thread.currentThread(), threadCount);
		Assert.isTrue(threadCount.intValue() < 5, "An unexpected thread count record: 5 >= " + threadCount);

		try {
			Thread.sleep(500L);
		} catch (InterruptedException ie) {
			// suppress
		}

		Assert.isTrue(threadCount.intValue() == 5, "An unexpected thread count record: 5 != " + threadCount);
	}
	
	@Override
	protected void onShutdown(ApplicationEvent event) {
	}
	
	@Threaded(threads = 5)
	private void try5(Thread parentThread, MutableInt threadCount) {
		this.logger.info("Running inside thread: {}", Thread.currentThread().getId());
		Assert.isTrue(parentThread.getId() != Thread.currentThread().getId(), "An unexpected thread ID: " + Thread.currentThread().getId());
		
		try {
			Thread.sleep(this.random.nextInt(100) + 100L);
			threadCount.increment();
		} catch (InterruptedException ie) {
			// suppress
		}
	}
	
	@Threaded(threads = 5, join = true)
	private void try5sync(Thread parentThread, MutableInt threadCount) {
		this.logger.info("Running inside thread: {}", Thread.currentThread().getId());
		Assert.isTrue(parentThread.getId() != Thread.currentThread().getId(), "An unexpected thread ID: " + Thread.currentThread().getId());

		try {
			Thread.sleep(this.random.nextInt(100) + 100L);
			threadCount.increment();
		} catch (InterruptedException ie) {
			// suppress
		}
	}

}
