package com.inteligr8.alfresco.annotations.aspect;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.DeclarePrecedence;
import org.aspectj.lang.annotation.Pointcut;

import jakarta.transaction.Transactional;

/**
 * @see jakarta.transaction.Transactional
 */
@Aspect
@DeclarePrecedence("com.inteligr8.alfresco.annotations.aspect.AuthorizedAspect, com.inteligr8.alfresco.annotations.aspect.RetryingTransactionAspect")
public class RetryingTransactionAspect extends AbstractRetryingTransactionAspect {
	
	@Override
	public String getJtaInterfaceName() {
		return Transactional.class.getName();
	}
	
	@Pointcut("@annotation(org.springframework.transaction.annotation.Transactional) && execution(* *(..))")
	public void isTransactionalAnnotated() {
	}
	
	@Pointcut("@annotation(jakarta.transaction.Transactional) && execution(* *(..))")
	public void isJtaTransactionalAnnotated() {
	}
	
	@Pointcut("@annotation(com.inteligr8.alfresco.annotations.TransactionalRetryable) && execution(* *(..))")
	public void isTransactionalRetryableAnnotated() {
	}
	
	@Around("isTransactionalAnnotated() || isJtaTransactionalAnnotated() || isTransactionalRetryableAnnotated()")
	public Object retryingTransactional(ProceedingJoinPoint joinPoint) throws Throwable {
		return super.retryingTransactional(joinPoint);
	}

}
