package com.inteligr8.alfresco.annotations.service.impl;

import javax.transaction.xa.XAResource;

import org.alfresco.repo.transaction.AlfrescoTransactionSupport;
import org.alfresco.repo.transaction.TransactionListener;

import jakarta.transaction.HeuristicMixedException;
import jakarta.transaction.HeuristicRollbackException;
import jakarta.transaction.NotSupportedException;
import jakarta.transaction.RollbackException;
import jakarta.transaction.Status;
import jakarta.transaction.Synchronization;
import jakarta.transaction.SystemException;
import jakarta.transaction.Transaction;
import jakarta.transaction.UserTransaction;

public class AlfrescoTransaction implements UserTransaction, Transaction {
	
	private UserTransaction userTx;
	
	public AlfrescoTransaction(UserTransaction userTx) {
		this.userTx = userTx;
	}
	
	@Override
	public void begin() throws NotSupportedException, SystemException {
		this.userTx.begin();
	}
	
	@Override
	public void commit() throws RollbackException, HeuristicMixedException, HeuristicRollbackException,
			SecurityException, IllegalStateException, SystemException {
		this.userTx.commit();
	}
	
	@Override
	public boolean delistResource(XAResource xaRes, int flag) throws IllegalStateException, SystemException {
		return false;
	}
	
	@Override
	public boolean enlistResource(XAResource xaRes) throws RollbackException, IllegalStateException, SystemException {
		return false;
	}
	
	@Override
	public int getStatus() throws SystemException {
		return this.userTx.getStatus();
	}
	
	@Override
	public void registerSynchronization(Synchronization sync)
			throws RollbackException, IllegalStateException, SystemException {
		AlfrescoTransactionSupport.bindListener(new TransactionListener() {
			@Override
			public void flush() {
			}
			
			@Override
			public void beforeCompletion() {
				sync.beforeCompletion();
			}
			
			@Override
			public void beforeCommit(boolean readOnly) {
			}
			
			@Override
			public void afterRollback() {
				sync.afterCompletion(Status.STATUS_ROLLEDBACK);
			}
			
			@Override
			public void afterCommit() {
				sync.afterCompletion(Status.STATUS_COMMITTED);
			}
		});
	}
	
	@Override
	public void rollback() throws IllegalStateException, SecurityException, SystemException {
		this.userTx.rollback();
	}
	
	@Override
	public void setRollbackOnly() throws IllegalStateException, SystemException {
		this.userTx.setRollbackOnly();
	}
	
	@Override
	public void setTransactionTimeout(int seconds) throws SystemException {
		this.userTx.setTransactionTimeout(seconds);
	}

}
