package com.inteligr8.alfresco.annotations.service.impl;

import java.util.function.Supplier;

import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.UserTransaction;

import org.alfresco.service.transaction.TransactionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * This bean implements a standard TransactionManager for ACS. 
 */
@Component
public class AlfrescoTransactionManager implements TransactionManager {
	
	@Autowired
	private TransactionService txService;
	
	private ThreadLocal<AlfrescoTransaction> tx = ThreadLocal.withInitial(new Supplier<AlfrescoTransaction>() {
		@Override
		public AlfrescoTransaction get() {
			return null;
		}
	});
	
	@Override
	public void begin() throws NotSupportedException, SystemException {
		UserTransaction userTx = this.txService.getNonPropagatingUserTransaction();
		AlfrescoTransaction tx = new AlfrescoTransaction(userTx);
		tx.begin();
		this.tx.set(tx);
	}
	
	@Override
	public void commit() throws RollbackException, HeuristicMixedException, HeuristicRollbackException,
			SecurityException, IllegalStateException, SystemException {
		AlfrescoTransaction tx = this.tx.get();
		if (tx == null)
			throw new IllegalStateException();
		
		tx.commit();
	}
	
	@Override
	public int getStatus() throws SystemException {
		AlfrescoTransaction tx = this.tx.get();
		if (tx == null)
			return Status.STATUS_NO_TRANSACTION;
		
		return tx.getStatus();
	}
	
	@Override
	public Transaction getTransaction() throws SystemException {
		return this.tx.get();
	}
	
	@Override
	public void resume(Transaction tx) throws InvalidTransactionException, IllegalStateException, SystemException {
		if (!(tx instanceof AlfrescoTransaction))
			throw new InvalidTransactionException("An AlfrescoTransaction is expected; received: " + tx.getClass());
		if (this.tx.get() != null)
			throw new IllegalStateException();
		
		this.tx.set((AlfrescoTransaction) tx);
	}
	
	@Override
	public void rollback() throws IllegalStateException, SecurityException, SystemException {
		AlfrescoTransaction tx = this.tx.get();
		if (tx == null)
			throw new IllegalStateException();
		
		tx.rollback();
	}
	
	@Override
	public void setRollbackOnly() throws IllegalStateException, SystemException {
		AlfrescoTransaction tx = this.tx.get();
		if (tx == null)
			throw new IllegalStateException();
		
		tx.setRollbackOnly();
	}
	
	@Override
	public void setTransactionTimeout(int seconds) throws SystemException {
		AlfrescoTransaction tx = this.tx.get();
		if (tx != null)
			tx.setTransactionTimeout(seconds);
	}
	
	@Override
	public Transaction suspend() throws SystemException {
		try {
			return this.tx.get();
		} finally {
			this.tx.set(null);
		}
	}

}
