package com.inteligr8.alfresco.annotations.util;

import javax.transaction.Transactional;

import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;

public class JavaxTransactionalAnnotationAdapter implements JtaTransactionalAnnotationAdapter {
	
	public static final String JTA_INTERFACE_NAME = "javax.transaction.Transactional";
	
	private final Transactional txl;
	
	public static JavaxTransactionalAnnotationAdapter cast(Object obj) {
		return new JavaxTransactionalAnnotationAdapter((Transactional) obj);
	}
	
	public JavaxTransactionalAnnotationAdapter(Transactional txl) {
		this.txl = txl;
	}

	@Override
	public boolean isReadOnly() {
		return false;
	}

	@Override
	public Propagation getPropagation() {
		switch (this.txl.value()) {
			case MANDATORY:
				return Propagation.MANDATORY;
			case REQUIRED:
				return Propagation.REQUIRED;
			case REQUIRES_NEW:
				return Propagation.REQUIRES_NEW;
			case SUPPORTS:
				return Propagation.SUPPORTS;
			case NOT_SUPPORTED:
				return Propagation.NOT_SUPPORTED;
			case NEVER:
				return Propagation.NEVER;
			default:
				throw new IllegalStateException("This should never happen");
		}
	}

	@Override
	public Isolation getIsolation() {
		return Isolation.DEFAULT;
	}

	@Override
	public int getTimeoutInSeconds() {
		return 0;
	}

	@SuppressWarnings("unchecked")
	@Override
	public Class<? extends Throwable>[] getRollbackFor() {
		return this.txl.rollbackOn();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Class<? extends Throwable>[] getNoRollbackFor() {
		return this.txl.dontRollbackOn();
	}

}
